@@ -13,9 +13,8 @@
 #'
 #' @export
 #' @family chatbots
-#' @param api_key The API key to use for authentication. You generally should
-#'   not supply this directly, but instead set the `PERPLEXITY_API_KEY` environment
-#'   variable.
+#' @param api_key `r api_key_param("PERPLEXITY_API_KEY")`
+#' @param model `r param_model("llama-3.1-sonar-small-128k-online")`
 #' @inheritParams chat_openai
 #' @inherit chat_openai return
 #' @examples
@@ -23,20 +22,19 @@
 #' chat <- chat_perplexity()
 #' chat$chat("Tell me three jokes about statisticians")
 #' }
-chat_perplexity <- function(system_prompt = NULL,
-                            turns = NULL,
-                            base_url = "https://api.perplexity.ai/",
-                            api_key = perplexity_key(),
-                            model = NULL,
-                            seed = NULL,
-                            api_args = list(),
-                            echo = NULL) {
-
+chat_perplexity <- function(
+  system_prompt = NULL,
+  base_url = "https://api.perplexity.ai/",
+  api_key = perplexity_key(),
+  model = NULL,
+  seed = NULL,
+  api_args = list(),
+  echo = NULL
+) {
   model <- set_default(model, "llama-3.1-sonar-small-128k-online")
 
   chat_openai(
     system_prompt = system_prompt,
-    turns = turns,
     base_url = base_url,
     api_key = api_key,
     model = model,
